#
# address.rb
#
#   Copyright (c) 1999 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#
#   This program is free software.
#   you can distribute/modify this program under the terms of
#   the GNU Lesser General Public License version 2 or later.
#

require 'amstd/errutil'


module EST


  class ModalTask

    def initialize( book )
      @pipe = Queue.new
      @book = book
      construct_gui
    end

    def invoke( block )
      @win.show_all
      Thread.fork do
        ret = @pipe.pop
        @win.hide_all
        if ret then
          block.call @work.call
        else
          block.call nil
        end
      end
    end

    private

    def task_end( result )
      @pipe.push result
    end

  end


  class EditAddress < ModalTask

    def invoke( block )
      reset
      super
    end


    private


    def strarr2list( arr )
      arr.filter do |i|
        li = Gtk::ListItem.new( i )
        li.show
        li
      end
      arr
    end

    def reset
      @list.clear
      entries = @book.values.sort{|a,b| a.name <=> b.name }
      entries.each_with_index do |ent, idx|
        @list.append [ent.name, ent.properties.join(','), ent.addr]
        @list.set_row_data idx, ent
      end

      @proplist.clear_items( 0, -1 )

      @allproplist.clear_items( 0, -1 )
      @allproplist.append_items strarr2list( @book.properties.sort )
      
      if ent = entries[0] then
        @cur_row = 0
        @cur_ent = ent
        write_in
      end
    end


    def write_in
      @name_entry.set_text @cur_ent.name
      @address_entry.set_text @cur_ent.addr
      @proplist.clear_items( 0, -1 )
      @proplist.append_items strarr2list( @cur_ent.properties )
    end

    def write_back
      @cur_ent.name = nm = @name_entry.get_text
      @cur_ent.addr = ad = @address_entry.get_text

      arr = @cur_ent.properties
      @list.set_text @cur_row, 0, nm
      @list.set_text @cur_row, 1, arr.join(',')
      @list.set_text @cur_row, 2, ad
    end


    def set_callback
      @win.signal_connect( 'destroy' ) do |w|
        task_end false
      end

      @list.signal_connect( 'select_row' ) do |w, row, col, eve|
        write_back
        @cur_row = row
        @cur_ent = @list.get_row_data( row )
        write_in
      end

      @del_button.signal_connect( 'clicked' ) do |w|
        sel = @proplist.selection
        unless sel.empty? then
          sel.each do |i|
            @cur_ent.del_property i.child.get_text
          end
          @proplist.remove_items sel
        end
      end

      @add_button.signal_connect( 'clicked' ) do |w|
        sel = @allproplist.selection
        unless sel.empty? then
          sel.each do |i|
            @cur_ent.add_property i.child.get_text
          end
          @proplist.clear_items 0, -1
          @proplist.append_items strarr2list( @cur_ent.properties )
        end
      end

      @ok_button.signal_connect( 'clicked' ) do |w|
        task_end false #true
        false
      end

      @cancel_button.signal_connect( 'clicked' ) do |w|
        task_end false
        false
      end
    end


    def construct_gui
      @win = Gtk::Window.new( Gtk::WINDOW_TOPLEVEL )
      @win.set_modal true
      @win.set_title 'mailest address book'

      @box = Gtk::VBox.new( false, 0 )
      @win.add @box

      @message = Gtk::Label.new( 'choose data to edit' )
      @message.set_alignment 0, 0.5
      @box.pack_start @message, false, false, 5

      @listbox = Gtk::HBox.new( false, 0 )
      @box.pack_start @listbox, true, true, 0

      @list = Gtk::CList.new( ['name', 'property', 'address'] )
      @list.set_column_width 0, 150
      @list.set_column_width 1, 300
      @list.set_column_width 2, 150
      @listbox.pack_start @list, true, true, 0

      @listvadj = Gtk::Adjustment.new( 0, 0, 0, 0, 0, 0 )
      @listvscr = Gtk::VScrollbar.new( @listvadj )
      @list.vadjustment = @listvadj
      @listbox.pack_start @listvscr, false, false, 0

      @name_entry = Gtk::Entry.new
      @name_entry.set_editable true
      @box.pack_start @name_entry, false, false, 0
        
      @address_entry = Gtk::Entry.new
      @address_entry.set_editable true
      @box.pack_start @address_entry, false, false, 0

      @bottom_box = Gtk::HBox.new( false, 0 )
      @box.pack_start @bottom_box, true, true, 0


      @plist_box = Gtk::VBox.new( false, 0 )
      @bottom_box.pack_start @plist_box, true, true, 0
      @plist_box_in = Gtk::HBox.new( false, 0 )
      @plist_box.pack_start @plist_box_in, true, true, 0

      @allplist_box = Gtk::VBox.new( false, 0 )
      @bottom_box.pack_start @allplist_box, true, true, 0
      @allplist_box_in = Gtk::HBox.new( false, 0 )
      @allplist_box.pack_start @allplist_box_in, true, true, 0

      @proplist = Gtk::List.new
      @proplist.set_selection_mode Gtk::SELECTION_BROWSE
      @plist_box_in.pack_start @proplist, true, true, 0
      @proplist_scr = Gtk::VScrollbar.new
      @plist_box_in.pack_start @proplist_scr, false, false, 0
      @del_button = Gtk::Button.new( 'del' )
      @plist_box.pack_start @del_button, false, false, 0

      @allproplist = Gtk::List.new
      @allproplist.set_selection_mode Gtk::SELECTION_BROWSE
      @allplist_box_in.pack_start @allproplist, true, true, 0
      @allproplist_scr = Gtk::VScrollbar.new
      @allplist_box_in.pack_start @allproplist_scr, false, false, 0
      @add_button = Gtk::Button.new( 'add' )
      @allplist_box.pack_start @add_button, false, false, 0


      @button_box = Gtk::VBox.new( true, 0 )
      @bottom_box.pack_start @button_box, false, false, 0

      @ok_button = Gtk::Button.new( 'edit done' )
      @button_box.pack_start @ok_button, false, false, 2

      @cancel_button = Gtk::Button.new( 'cancel' )
      @button_box.pack_start @cancel_button, false, false, 2

      set_callback
    end

  end


  class GetAddress < ModalTask

    def invoke( block )
      @to = []
      @cc = []
      @bcc = []
      return super
    end


    def construct_gui
      @win = Gtk::Window.new( Gtk::WINDOW_TOPLEVEL )
      @win.set_modal true
      @win.signal_connect( 'destroy' ) do |w|
        task_end false
      end

      @box = Gtk::HBox.new( false, 0 )
      @win.add @box

      @list = Gtk::CList.new( ['', 'name', 'property'] )
      @list.set_column_width 0, 50
      @list.set_column_width 1, 150
      @list.set_column_width 2, 300

      @book.values.sort{|a,b| a.name <=> b.name }.each do |ent|
        @list.append ['', ent.name, ent.keys.join(',')]
        @list.set_row_data @list.rows - 1, ent.addr
      end

      @list.signal_connect( 'select_row' ) do |w, row, col, eve|
        ent = @list.get_row_data( row )

        sel = @to
        pix = 'To' #@to_pix
        if    @to_button.active? then  sel = @to  ; pix = 'To'
        elsif @cc_button.active? then  sel = @cc  ; pix = 'CC'
        elsif @bcc_button.active? then sel = @bcc ; pix = 'BCC'
        end

        sel.push ent
        [@to, @cc, @bcc].each {|arr| arr.delete ent unless arr == sel }
        # @list.set_pixmap row, 0, pix
        @list.set_text row, 0, pix
      end

      @list.signal_connect( 'unselect_row' ) do |w, row, col, eve|
        @list.set_text row, 0, ''
        ent = @list.get_row_data( row )
        [@to, @cc, @bcc].each do |arr|
          if arr.delete ent then
            break
          end
        end
      end

      @box.pack_start @list, false, false, 0


      @console_box = Gtk::VBox.new( false, 0 )
      @box.pack_start @console_box

      @message = Gtk::Label.new( 'select To/Cc/Bcc' )
      @console_box.pack_start @message, false, false, 5

      @to_button  = Gtk::RadioButton.new( nil, 'To' )
      @cc_button  = Gtk::RadioButton.new( @to_button.group, 'Cc' )
      @bcc_button = Gtk::RadioButton.new( @to_button.group, 'Bcc' )
      @console_box.pack_start @to_button,  false, false, 3
      @console_box.pack_start @cc_button,  false, false, 3
      @console_box.pack_start @bcc_button, false, false, 3

      @ok_button = Gtk::Button.new( 'Selection OK' )
      @ok_button.signal_connect( 'clicked' ) do |w|
        task_end false #true
        false
      end
      @console_box.pack_start @ok_button, false, false, 2

      @cancel_button = Gtk::Button.new( 'Cancel to write a mail' )
      @cancel_button.signal_connect( 'clicked' ) do |w|
        task_end false
        false
      end
      @console_box.pack_start @cancel_button, false, false, 2
    end

    private :construct_gui

  end


end   # module EST
