#
# readrc.rb
#
#   mailest resource reader
#
#   Copyright(c) 1999 Minero Aoki
#   aamine@dp.u-netsurf.ne.jp
#


require 'est/confp'


module EST


class ConfigError < StandardError; end


class RcReader

  def RcReader.read( fn )
    ent = ConfigFileParser.parse( fn )
    reader = new
    reader.configure ent
    reader.configure_if ent['interface'][ ::EST::Interface ]

    ent['first-profile']
  end

  ClassList = %w(
    smtp-server
    pop-server
    mailbox
    composer
    refile
    getter
    profile
  )

  def configure( entries )
    ClassList.each do |ccls|
puts "configuring #{ccls}..."
      mname = 'setup_' + ccls.gsub( '-', '_' )
      send mname, entries[ccls]
    end
  end

  # abstract configure_if


  def setup_smtp_server( config )
    config.each do |name, rc|
      CORE.sender.add name, rc
    end
  end

  def setup_pop_server( config )
    config.each do |name, rc|
      CORE.popper.add name, rc
    end
  end

  def setup_mailbox( config )
    config.each do |name, rc|
      CORE.mailbox.add_mbox name, rc
    end
  end

  def setup_composer( config )
    config.each do |name, rc|
      CORE.composer.add name, rc
    end
  end

  def setup_refile( config )
    return unless config

    config.each do |name, rc|
      rule = rc['rule']
unless rule
p rule
bug!
end
      rule.each do |ent|
        # ent = array of [ header, refile_regex, dest_mbox ]
        ent[1] = Regexp.new( ent[1] )
      end
      CORE.refiler.add name, rc
    end
  end

  def setup_profile( config )
    config.each do |name, rc|
      CORE.profile.add name, rc
    end
  end

  def confirm_config( profnm, profiles )
    ent = profiles[ profnm ]
    unless ent['mailbox'] and
           ent['smtp-server'] and
           ent['pop-server'] and
           ent['composer']
      CORE.warn "EST Warning: profile #{profnm} not exist: delete entry"
      profiles.delete profnm
    end
  end

  def setup_getter( config )
    config.each do |nm, rc|
      CORE.getter.add nm, rc
    end
  end

end

end   # module EST
